@@ -67,6 +67,8 @@ typedef	struct
 	char 	*cookie;		/* Erf/TS4 "cookie" value */
 	int	save_channels;		/* True if abnormal connection */
 	int	closing;		/* True if close_server called */
+	int	reconnect_to;		/* Server to connect to on EOF */
+	char	*quit_message;		/* Where we stash a quit message */
 }	Server;
 extern	Server	*server_list;
 #endif	/* NEED_SERVER_LIST */
@@ -118,7 +120,6 @@ extern	int	parsing_server_index;
 	int	sent_to_server			(int);
 
 	int	connect_to_new_server		(int, int, int);
-	int	get_connected 			(int, int);
 	int	close_all_servers		(const char *);
 	void	close_server			(int, const char *);
 	void	save_server_channels		(int);
@@ -149,6 +150,8 @@ const 	char *	get_server_version_string	(int);
 	int	is_server_connected		(int);
 	void	server_is_connected		(int, int);
 	int	auto_reconnect_callback		(void *);
+	int	server_reconnects_to		(int, int);
+	int	reconnect			(int);
 
 	int	get_server_port			(int);
 	int	get_server_local_port		(int);
@@ -185,5 +188,8 @@ const	char *	get_server_redirect		(int);
 
 	void	server_did_rejoin_channels	(int);
 	int	did_server_rejoin_channels	(int);
+	int     set_server_quit_message 	(int, const char *message);
+const char *    get_server_quit_message		(int);
+
 
 #endif /* _SERVER_H_ */
