@@ -13,19 +13,19 @@
 /*
  * irc_version is what $J returns, its the common-name for the version.
  */
-const char irc_version[] = "EPIC4-0.9.14";
-const char useful_info[] = "epic4 0 9 14";
+const char irc_version[] = "EPIC4-0.9.16";
+const char useful_info[] = "epic4 0 9 16";
 
 /*
  * internal_version is what $V returns, its the integer-id for the
  * version, and corresponds to the date of release, YYYYMMDD.
  */ 
-const char internal_version[] = "20001208";
+const char internal_version[] = "20010205";
 
 /*
  * In theory, this number is incremented for every commit.
  */
-const unsigned long	commit_id = 9;
+const unsigned long	commit_id = 37;
 
 /*
  * As a way to poke fun at the current rage of naming releases after
@@ -33,7 +33,7 @@ const unsigned long	commit_id = 9;
  * reality, I have decided to start doing that with EPIC.  These names
  * are intentionally and maliciously silly.  Complaints will be ignored.
  */
-const char ridiculous_version_name[] = "Perniscious";
+const char ridiculous_version_name[] = "Ostensible";
 
 #define __need_putchar_x__
 #include "status.h"
@@ -755,15 +755,13 @@ void	io (const char *what)
 static	const	char	*caller[51] = { NULL }; /* XXXX */
 	static	int	first_time = 1,
 			level = 0;
-static	struct	timeval cursor_timeout,
-			clock_timeout,
+static	struct	timeval	clock_timeout,
 			right_away,
 			timer,
 			*timeptr = NULL;
 		int	hold_over;
 		int	dccs;
 		fd_set	rd;
-		Screen	*screen;
 
 	static	int	old_level = 0;
 	static	int	last_warn = 0;
@@ -801,10 +799,6 @@ static	struct	timeval cursor_timeout,
 	{
 		first_time = 0;
 
-		/* time before cursor jumps from display area to input line */
-		cursor_timeout.tv_usec = 0L;
-		cursor_timeout.tv_sec = 1L;
-
 		/*
 		 * time delay for updating of internal clock
 		 *
@@ -885,17 +879,13 @@ static	struct	timeval cursor_timeout,
 		dcc_check(&rd);		/* XXX HACK XXX */
 
 	ExecuteTimers();
-
 	get_child_exit(-1);
+	if (level == 1 && need_defered_commands)
+		do_defered_commands();
 
-	if (!hold_over)
-		cursor_to_input();
+	cursor_to_input();
 	timeptr = &clock_timeout;
 
-	for (screen = screen_list; screen; screen = screen->next)
-		if (screen->alive && is_cursor_in_display(screen))
-			timeptr = &cursor_timeout;
-
 	if (update_clock(RESET_TIME))
 	{
 		/* 
@@ -919,6 +909,11 @@ static	struct	timeval cursor_timeout,
 	alloca(0);
 	caller[level] = NULL;
 	level--;
+
+#ifdef DELAYED_FREES
+	if (level == 0 && need_delayed_free)
+		do_delayed_frees();
+#endif
 	return;
 }
 
@@ -1258,7 +1253,7 @@ int 	main (int argc, char *argv[])
 	if (dont_connect)
 		display_server_list();		/* Let user choose server */
 	else
-		get_connected(0, -1);		/* Connect to default server */
+		reconnect(-1);			/* Connect to default server */
 
 	time(&idle_time);
 	set_input(empty_string);
