@@ -17,6 +17,7 @@
 #include "output.h"
 #include "screen.h"
 #include "window.h"
+#include "vars.h"
 #include "server.h"
 #include "list.h"
 #include "hook.h"
@@ -71,7 +72,7 @@ struct timeval		join_time;	/* When we joined the channel */
  * The variable "mode_str" must correspond in order to the modes defined
  * here, or all heck will break loose.  You have been warned.
  */
-static	char	mode_str[] = "aciklmnprstzR";
+static	char	mode_str[] = "aciklmnprstzOR";
 
 const int	MODE_ANONYMOUS	= 1 << 0;	/* av2.9 */
 const int	MODE_C		= 1 << 1;	/* erf/TS4 */
@@ -81,11 +82,12 @@ const int	MODE_LIMIT	= 1 << 4;	/* RFC */
 const int 	MODE_MODERATED	= 1 << 5;	/* RFC */
 const int	MODE_MSGS	= 1 << 6;	/* RFC */
 const int	MODE_PRIVATE	= 1 << 7;	/* RFC */
-const int	MODE_REGISTERED = 1 << 8;	/* Dalnet */
+const int	MODE_REGISTERED = 1 << 8;	/* Duhnet */
 const int	MODE_SECRET	= 1 << 9;	/* RFC */
 const int	MODE_TOPIC	= 1 << 10;	/* RFC */
 const int	MODE_Z		= 1 << 11;	/* erf/TS4 */
-const int	MODE_RESTRICTED = 1 << 12;	/* Dalnet */
+const int	MODE_OPER_ONLY	= 1 << 12;	/* Duhnet */
+const int	MODE_RESTRICTED = 1 << 13;	/* Duhnet */
 
 
 /* channel_list: list of all the channels you are currently on */
@@ -262,6 +264,12 @@ static void 	clear_channel (Channel *chan)
 /* Channel destructor -- caller must free "chan". */
 static void 	destroy_channel (Channel *chan)
 {
+	Channel	*tmp;
+	char *	old_from = NULL;
+	char *	new_to = NULL;
+	int	old_refnum = 0;
+	int	new_refnum = 0;
+
 	if (chan != channel_list)
 	{
 		if (!chan->prev)
@@ -278,33 +286,41 @@ static void 	destroy_channel (Channel *chan)
 	if (chan->next)
 		chan->next->prev = chan->prev;
 
-	if (chan->window)
+	if (chan->window && chan->window->current_channel &&
+		!my_stricmp(chan->channel, chan->window->current_channel))
 	{
-		/*
-		 * If 'chan' is a current channel (and believe me, there is
-		 * no easy way to tell short of looking), then we will want
-		 * to give the window a chance to do something about it.  So
-		 * we call channel_going_away to tell the window that this 
-		 * channel's days are numbered.  If this is a current channel
-		 * then it will reset it's current channel to some other
-		 * channel it has, or to nothing, as the case may be.  It is
-		 * customary for the window to arrange for this channel's
-		 * window to be set to NULL if this was indeed the current
-		 * channel, so we *still* have to test to make sure that this
-		 * channel is not the current channel when this returns (ugh).
-		 */
-		channel_going_away(chan->window, chan->channel);
+		old_from = m_strdup(chan->window->current_channel);
+		old_refnum = chan->window->refnum;
+		new_free(&chan->window->current_channel);
+		chan->window->update |= UPDATE_STATUS;
 
-		/*
-		 * So if 'chan->window' is not null, then this was probably
-		 * not a current channel.  But if 'current_channel' exists,
-		 * and it's pointing at us, look out!  If this ever does
-		 * happen, we take matters into our own hands and tell the
-		 * window to get lost.
-		 */
-		if (chan->window && chan->window->current_channel)
-		  if (!my_stricmp(chan->window->current_channel, chan->channel))
-			set_channel_by_refnum(chan->window->refnum, NULL);
+		if (get_int_var(SWITCH_CHANNEL_ON_PART_VAR))
+		{
+		    for (tmp = channel_list; tmp; tmp = tmp->next)
+		    {
+			if (tmp->window != chan->window || 
+					tmp->server != chan->server)
+				continue;
+
+			new_to = m_strdup(tmp->channel);
+			new_refnum = tmp->window->refnum;
+			malloc_strcpy(&chan->window->current_channel, 
+					tmp->channel);
+
+			/*
+			 * Remove "waiting channel" if we're waiting for 
+			 * this channel.  ;-)
+			 */
+			if (chan->window->waiting_channel && 
+				!my_stricmp(tmp->channel, 
+						chan->window->waiting_channel))
+				new_free(&chan->window->waiting_channel);
+
+			set_channel_window(chan->window, 
+					chan->window->current_channel);
+			break;
+		     }
+		}
 	}
 
 	new_free(&chan->channel);
@@ -321,6 +337,19 @@ static void 	destroy_channel (Channel *chan)
 	new_free(&chan->key); 
 	chan->chop = 0;
 	chan->voice = 0;
+
+	if (old_from)
+	{
+		do_hook(SWITCH_CHANNELS_LIST, "%d %s %s",
+				old_refnum, old_from, zero);
+		new_free(&old_from);
+	}
+	if (new_to)
+	{
+		do_hook(SWITCH_CHANNELS_LIST, "%d %s %s",
+				new_refnum, zero, new_to);
+		new_free(&new_to);
+	}
 }
 
 /*
@@ -898,6 +927,9 @@ static void	decifer_mode (const char *modes, Channel *chan)
 				add = 0;
 				continue;
 
+			case 'a':
+				value = MODE_ANONYMOUS;
+				break;
 			case 'c':
 				value = MODE_C;
 				break;
@@ -928,6 +960,9 @@ static void	decifer_mode (const char *modes, Channel *chan)
 			case 'z':		/* Erf/TS4 "zapped" */
 				value = MODE_Z;
 				break;
+			case 'O':		/* Duhhnet's oper-only */
+				value = MODE_OPER_ONLY;
+				break;
 			case 'k':
 			{
 				char *key;
@@ -1335,6 +1370,7 @@ void 	reconnect_all_channels (void)
 {
 	Channel *tmp = NULL;
 	char	*channels = NULL;
+	char	*keyed_channels = NULL;
 	char	*keys = NULL;
 
 	while (traverse_all_channels(&tmp, from_server))
@@ -1344,9 +1380,14 @@ void 	reconnect_all_channels (void)
 			     "but it isn't inactive!", 
 				tmp->channel, tmp->server);
 
-		/* Ugh.  Hybrid makes me do this. */
-		m_s3cat(&channels, ",", tmp->channel);
-		m_s3cat(&keys, ",", tmp->key ? tmp->key : empty_string);
+		if (tmp->key)
+		{
+			m_s3cat(&keyed_channels, ",", tmp->channel);
+			m_s3cat(&keys, ",", tmp->key);
+		}
+		else
+			m_s3cat(&channels, ",", tmp->channel);
+
 		clear_channel(tmp);
 	}
 
@@ -1357,11 +1398,13 @@ void 	reconnect_all_channels (void)
 	 * Interestingly enough, black magic on the server's part makes
 	 * this work.  I sure hope they dont "break" this in the future...
 	 */
+	if (keyed_channels)
+		send_to_server("JOIN %s %s", keyed_channels, keys);
 	if (channels)
-		send_to_server("JOIN %s %s", channels, 
-					keys ? keys : empty_string);
+		send_to_server("JOIN %s", channels);
 
 	new_free(&channels);
+	new_free(&keyed_channels);
 	new_free(&keys);
 
 	/*
@@ -1492,6 +1535,92 @@ void	unset_window_current_channel (Window *old_w, Window *new_w)
 	}
 }
 
+/*
+ * This moves "chan" from "old_w" to "new_w", safely -- this replaces the old
+ * way of "unset_window_current_channel" and "reset_window_current_channel".
+ * This was written by Robohak in January 2001.
+ */
+void   move_channel_to_window (const char *chan, Window *old_w, Window *new_w)
+{
+	Channel *tmp = NULL;
+	char *	old_chan = NULL;
+	int	reset_old_w = 0;
+	int	found = 0;
+
+	if (!old_w || !old_w->current_channel || !new_w ||
+			old_w->server < 0 || !chan || !strcmp(chan, zero))
+	       return;
+
+	while (traverse_all_channels(&tmp, old_w->server))
+	{
+	    if (tmp->window == old_w && !my_stricmp(chan, tmp->channel))
+	    {
+		if (!my_stricmp(chan, old_w->current_channel))
+		{
+			new_free(&old_w->current_channel);
+			old_w->update |= UPDATE_STATUS;
+
+			    if (new_w->current_channel)
+				old_chan = m_strdup(new_w->current_channel);
+			    else
+				old_chan = NULL;
+
+			    malloc_strcpy(&new_w->current_channel, chan);
+
+			    /* 
+			     * Remove "waiting_channel" if we're waiting for 
+			     * this channel. ;-) 
+			     */
+			    if (new_w->waiting_channel && 
+				!my_stricmp(chan, new_w->waiting_channel))
+				    new_free(&new_w->waiting_channel);
+
+			    new_w->update |= UPDATE_STATUS;
+			    set_channel_window(new_w, new_w->current_channel);
+
+			reset_old_w = 1;
+			break;
+		}
+	    }
+	}
+
+	if (reset_old_w)
+	{
+	    tmp = NULL;
+	    while (traverse_all_channels(&tmp, old_w->server))
+	    {
+		if (tmp->window == old_w && my_stricmp(tmp->channel, chan))
+		{
+		    malloc_strcpy(&old_w->current_channel, tmp->channel);
+
+		    /* 
+		     * Remove "waiting_channel" if we're waiting for this 
+		     * channel. ;-) 
+		     */
+		    if (old_w->waiting_channel && !my_stricmp(tmp->channel, 
+							old_w->waiting_channel))
+			new_free(&old_w->waiting_channel);
+
+		    old_w->update |= UPDATE_STATUS;
+		    set_channel_window(old_w, old_w->current_channel);
+		    found = 1;
+		    break;
+		}
+	    }
+
+	    do_hook(SWITCH_CHANNELS_LIST, "%d %s %s", 
+			old_w->refnum, chan, zero);
+	    do_hook(SWITCH_CHANNELS_LIST, "%d %s %s", 
+			new_w->refnum, old_chan ? old_chan : zero, chan);
+	    if (found)
+		do_hook(SWITCH_CHANNELS_LIST, "%d %s %s", old_w->refnum, zero,
+		       old_w->current_channel ? old_w->current_channel : zero);
+
+	    new_free(&old_chan);
+	}
+}
+
+
 /*
  * This attempts to find another current channel for a window when the 
  * current channel for a window is 'stolen' by another window.
@@ -1807,7 +1936,10 @@ void	channel_check_windows (void)
 			panic("Channel [%s] on server [%d] is inactive "
 				"even though this server is connected!",
 				tmp->channel, tmp->server);
-		if (!did_server_rejoin_channels(tmp->server) && !tmp->inactive)
+
+		if (is_server_connected(tmp->server) &&
+				!did_server_rejoin_channels(tmp->server) && 
+				!tmp->inactive)
 			panic("Channel [%s] on server [%d] is NOT inactive "
 				"even though this server is NOT connected!",
 				tmp->channel, tmp->server);
