@@ -10,7 +10,7 @@
  */
 
 #if 0
-static	char	rcsid[] = "$Id: numbers.c,v 1.2 2000/12/06 03:25:52 jnelson Exp $";
+static	char	rcsid[] = "$Id: numbers.c,v 1.6 2001/01/12 20:11:36 jnelson Exp $";
 #endif
 
 #include "irc.h"
@@ -201,6 +201,29 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		break;
 	}
 
+	case 10:		/* EFNext "Use another server"	010 */
+	{
+		char *	new_server;
+		int	new_port;
+		int	old_server = from_server;
+
+		if (!ArgList[0] || !ArgList[1] || !ArgList[2])
+			break;		/* Not what i'm expecting */
+
+		new_server = ArgList[0];
+		new_port = atoi(ArgList[1]);
+		PasteArgs(ArgList, 2);
+
+		if (do_hook(current_numeric, "%s %s %d %s", from, 
+				new_server, new_port, ArgList[2]))
+			display_msg(from, ArgList);
+
+		add_to_server_list(new_server, new_port, NULL, NULL, 0);
+		server_reconnects_to(old_server, from_server);
+		from_server = old_server;
+		break;
+	}
+
 	case 14:		/* Erf/TS4 "cookie" numeric	014 */
 		set_server_cookie(from_server, ArgList[0]);
 		break;
@@ -813,9 +836,9 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		{
 			char	server_num[8];
 
+			server_reconnects_to(from_server, -1);
 			say("Password required for connection to server %s",
 				get_server_name(from_server));
-			close_server(from_server, empty_string);
 			if (!dumb_mode)
 			{
 				strlcpy(server_num, ltoa(from_server), 8);
@@ -836,12 +859,7 @@ void 	numbered_command (char *from, int comm, char **ArgList)
 		 */
 		if (do_hook(current_numeric, "%s %s", from, ArgList[0]))
 			display_msg(from, ArgList);
-		close_server(from_server, empty_string);
-		window_check_servers();
-#ifndef I_DONT_TRUST_MY_USERS
-		if (from_server == primary_server)
-			get_connected(from_server + 1, from_server);
-#endif
+		server_reconnects_to(from_server, -1);
 		break;
 	}
 
