@@ -495,7 +495,8 @@ static void p_error (char *from, char **ArgList)
 	PasteArgs(ArgList, 0);
 	if (!ArgList[0])
 		{fake();return;}
-	say("%s", ArgList[0]);
+	if (do_hook(ERROR_LIST, "%s %s", from, ArgList[0]))
+		say("%s %s", from, ArgList[0]);
 }
 
 void	add_user_who (char *from, char **ArgList)
@@ -661,15 +662,11 @@ static void	p_kill (char *from, char **ArgList)
 	 * we do any output, in case we want the output to go anywhere
 	 * meaningful.
 	 */
-	save_server_channels(from_server);
-	close_server(from_server, empty_string);
-
 	if (strchr(from, '.'))
         {
 		say("Server [%s] has rejected you. (%s)",
 			from, ArgList[1] ? ArgList[1] : 
 			"probably due to a nick collision");
-		get_connected(from_server, from_server);
 	}
 	else
 	{
@@ -702,9 +699,9 @@ static void	p_kill (char *from, char **ArgList)
 				  auto_reconnect_callback,
 				  sc, NULL, current_window);
 		}
-	}
 
-	window_check_servers();
+		server_reconnects_to(from_server, -1);
+	}
 }
 
 static void p_ping (char *from, char **ArgList)
@@ -888,7 +885,7 @@ static void strip_modes (char *from, char *channel, char *line)
 
 			case 'a' : case 'i' : case 'm' : case 'n' :
 			case 'p' : case 's' : case 't' : case 'z' : 
-			case 'c' : case 'r' : case 'R' :
+			case 'c' : case 'r' : case 'R' : case 'O' :
 			{
 				do_hook(MODE_STRIPPED_LIST,
 					"%s %s %c%c",
