@@ -46,7 +46,7 @@ static void	clear_reconnect_counts (void);
 						 * is made */
 	int	connected_to_server = 0;	/* true when connection is
 						 * confirmed */
-
+	int	reconnects_to_hint = 0;		/* XXX Hack.  Don't remind me */
 
 
 /************************ SERVERLIST STUFF ***************************/
@@ -109,6 +109,9 @@ void 	add_to_server_list (const char *server, int port, const char *password, co
 		s->fudge_factor = 0;
 		s->registration_pending = 0;
 		s->resetting_nickname = 0;
+		s->reconnects = 0;
+		s->quit_message = NULL;
+		s->save_channels = 0;
 
 		if (password && *password)
 			malloc_strcpy(&s->password, password);
@@ -619,7 +622,12 @@ BUILT_IN_COMMAND(servercmd)
 
 		/* /SERVER + means go to the next server */
 		else
-			get_connected(from_server + 1, from_server);
+		{
+			set_server_quit_message(from_server, 
+					"Changing servers");
+			server_reconnects_to(from_server, from_server + 1);
+			reconnect(from_server);
+		}
 
 		window_check_servers();
 	}
@@ -639,11 +647,24 @@ BUILT_IN_COMMAND(servercmd)
 			    say("You can't close your primary server!");
 			    return;
 			}
-			server_list[i].save_channels = 0;
-			close_server(i, "closing server");
+
+			set_server_quit_message(from_server, 
+					"Disconnected at user request");
+			server_reconnects_to(i, -1);
+			reconnect(i);
 		}
 		else
-			get_connected(from_server - 1, from_server);
+		{
+			set_server_quit_message(from_server, 
+					"Changing servers");
+			if (from_server == 0)
+				server_reconnects_to(from_server, 
+							number_of_servers - 1);
+			else
+				server_reconnects_to(from_server, 
+							from_server - 1);
+			reconnect(from_server);
+		}
 
 		window_check_servers();
 	}
@@ -657,7 +678,8 @@ BUILT_IN_COMMAND(servercmd)
 
 		clear_reconnect_counts();
 		i = find_server_refnum(server, &args);
-		connect_to_new_server(i, j, 0);
+		server_reconnects_to(j, i);
+		reconnect(j);
 		window_check_servers();
 	}
 }
@@ -668,7 +690,7 @@ BUILT_IN_COMMAND(servercmd)
  * do_server: check the given fd_set against the currently open servers in
  * the server list.  If one have information available to be read, it is read
  * and and parsed appropriately.  If an EOF is detected from an open server,
- * we call get_connected() to try to keep that server connection alive.
+ * we call reconnect() to try to keep that server connection alive.
  */
 void	do_server (fd_set *rd)
 {
@@ -693,27 +715,14 @@ void	do_server (fd_set *rd)
 
 			case -1:	/* EOF or other error */
 			{
-				int	sopen = is_server_connected(i);
-
-				/* Always save channels */
-				save_server_channels(i);
-				close_server(i, empty_string);
+				server_is_connected(i, 0);
+				close_server(i, NULL);
 				say("Connection closed from %s: %s", 
 					server_list[i].name,
 					(dgets_errno == -1) ? 
 					     "Remote end closed connection" : 
 					     strerror(dgets_errno));
-
-				/*
-				 * If we were never connected to the server,
-				 * eg, they rejected us at registration time,
-				 * then we will just move on to the next
-				 * server in the list.
-				 */
-				if (sopen)
-					get_connected(i, i);
-				else
-					get_connected(i + 1, i);
+				reconnect(i);
 				i++;		/* NEVER DELETE THIS! */
 				break;
 			}
@@ -857,10 +866,12 @@ static void 	vsend_to_server (const char *format, va_list args)
 		    if (write(des, buffer, strlen(buffer)) == -1 && 
 			(!get_int_var(NO_FAIL_DISCONNECT_VAR)))
 		    {
-			server_list[server].save_channels = 1;
-			close_server(server, strerror(errno));
-			say("Write to server failed.  Closing connection.");
-			get_connected(server, server);
+			if (server_list[server].connected)
+			{
+			    server_is_connected(des, 0);
+			    say("Write to server failed.  Closing connection.");
+			    reconnect(server);
+			}
 		    }
 		}
 	}
@@ -981,6 +992,7 @@ static int 	connect_to_server (int new_server)
 	 */
 	message_from((char *) 0, LOG_CRAP);
 	update_all_status();
+	server_reconnects_to(new_server, new_server + 1);
 	return 0;			/* New connection established */
 }
 
@@ -1069,7 +1081,7 @@ int 	connect_to_new_server (int new_server, int old_server, int new_conn)
 			 * the windows wont be re-assigned on us.  This 
 			 * gives us a chance to do the right thing below.
 			 */
-			server_list[old_server].save_channels = 1;
+			save_server_channels(old_server);
 			close_server(old_server, "changing servers");
 		}
 
@@ -1125,27 +1137,34 @@ int 	connect_to_new_server (int new_server, int old_server, int new_conn)
 	return 0;
 }
 
-
 /*
- * 'get_connected' is a front end to 'connect_to_new_server' that repeatedly
- * tries as many servers as it takes to get a new server connection going.
- * If all else fails, it reverts back to the original connection, provided
- * that the original connection existed. ;-)
+ * This function supplants the "get_connected" function; it started out life
+ * as a front end to get_connected(), but it ended up being more flexible
+ * than get_connected() and everyone was calling reconnect() instead, so I 
+ * just rolled the two of them together and here we are.
  *
- * This is the lynchpin through which all of the auto-reconnect madness 
- * happens.  Because it is recursive, bad things can happen if a server
- * connection behaves in a way that we are not prepared to handle.  Therefore
- * the impetus is on this function to refuse to act and disconnect the user
- * and force manual intervention if it appears we are spinning out of control.
- * Because recursive autoreconnects can occur, we use the "reconnects" member
- * of the server structure to throttle unsuccessfull reconnection attempts 
- * here.  The number of unsuccessfull (defined as not completing registration)
- * attempts is controlled by /set max_reconnect_attempts.
+ * The purpose of this function is to make it easier to handle dropped
+ * server connections.  If we recieve a Notice Of Termination (such as an
+ * 010 or 465 numerics, or a KILL or such), we may not want to actually 
+ * terminate the connection before the server does as it may send us more
+ * important information.  Instead of closing the connection, we instead
+ * drop a "hint" to the server what it should do when it sees an EOF.  For
+ * most circumstances, the "hint" is "reconnect to yourself again".  However,
+ * sometimes we want to reconnect to a different server.  For all of these
+ * circumstances, this function is called to figure out what to do.
+ *
+ * Use the function "server_reconnects_to" to control how this function
+ * behaves.  Setting the reconnecting server to -1 inhibits any connection
+ * from occuring upon EOF.  Setting the reconnecting server to itself causes
+ * a normal reconnect act.  Setting the reconnecting server to another refnum
+ * causes the server connection to be migrated to that refnum.  The original
+ * connection is closed only if the new refnum is successfully opened.
  */
-int 	get_connected (int new, int old)
+int	reconnect (int oldserv)
 {
-	int	i;
-	int	j;
+	int	newserv;
+	int	i, j;
+	int	was_connected = 0;
 	int	connected;
 	int	max_reconnects = get_int_var(MAX_RECONNECTS_VAR);
 
@@ -1156,23 +1175,59 @@ int 	get_connected (int new, int old)
 		return -1;
 	}
 
-	if (new < 0)
-		new = number_of_servers - 1;		/* Sanity check */
+	if (oldserv >= 0 && oldserv < number_of_servers)
+	{
+		was_connected = is_server_connected(oldserv);
+		newserv = server_list[oldserv].reconnect_to;
 
-	connected = is_server_connected(old);
+		/*
+		 * Inhibit automatic reconnections.
+		 */
+		if (!was_connected && newserv == oldserv &&
+				get_int_var(AUTO_RECONNECT_VAR) == 0)
+			newserv = -1;
+
+		if (newserv != -1)
+			save_server_channels(oldserv);
+		if (newserv == oldserv || newserv == -1)
+			close_server(oldserv, get_server_quit_message(oldserv));
+		connected = is_server_connected(oldserv);
+	}
+	else
+	{
+		newserv = reconnects_to_hint;
+		connected = 0;
+	}
+
+	if (newserv < 0)
+		return -1;		/* User wants to disconnect */
 
 	/* Try all of the other servers, stop when one of them works. */
 	for (i = 0; i < number_of_servers; i++)
 	{
-		j = (i + new) % number_of_servers;
-		if (new != old && j == old)
+		j = (i + newserv) % number_of_servers;
+		if (newserv != oldserv && j == oldserv)
 			continue;
 		if ((server_list[j].reconnects++) > max_reconnects) {
 			say("Auto-reconnect has been throttled because too many unsuccessfull attempts to connect to server %d have been performed.", j);
 			break;
 		}
-		if (connect_to_new_server(j, old, 0) == 0)
+		if (connect_to_new_server(j, oldserv, 0) == 0)
 			return j;
+
+		/*
+		 * Because of the new way we handle things, if the old
+		 * server was connected, then this was because the user 
+		 * was trying to change to a new server; if the connection
+		 * to the new server failed, then we want to resume the old
+		 * connection.  But if the server was not connected, then 
+		 * that means we already saw an EOF on either the read or
+		 * write end, and so there is no connection to resume!  In
+		 * which case we just keep cycling our servers until we find
+		 * some place we like.
+		 */
+		if (connected)
+			break;
 	}
 
 	/* If we reach this point, we have failed.  Time to punt */
@@ -1181,21 +1236,20 @@ int 	get_connected (int new, int old)
 	 * If our prior state was connected, revert back to the prior
 	 * connected server.
 	 */
-	if (connected && new != old)
+	if (connected && newserv != oldserv)
 	{
-		say("All attempts to establish a new server connection "
-			"have failed.");
+		say("A new server connection could not be established.");
 		say("Your previous server connection will be resumed.");
-		from_server = old;
+		from_server = oldserv;
 		return -1;
 	}
 
 	/*
-	 * In any situation, if 'old' is not connected at this point, then
-	 * we need to throw away it's channels.
+	 * In any situation, if 'oldserv' is not connected at this point, 
+	 * then we need to throw away it's channels.
 	 */
-	if (!is_server_connected(old))
-		destroy_server_channels(old);
+	if (!is_server_connected(oldserv))
+		destroy_server_channels(oldserv);
 
 	/*
 	 * Our prior state was unconnected.  Tell the user
@@ -1214,8 +1268,9 @@ int 	close_all_servers (const char *message)
 
 	for (i = 0; i < number_of_servers; i++)
 	{
-		server_list[i].save_channels = 0;
-		close_server(i, message);
+		set_server_quit_message(i, message);
+		server_reconnects_to(i, -1);
+		reconnect(i);
 	}
 
 	return 0;
@@ -1554,7 +1609,7 @@ void 	password_sendline (char *data, char *line)
 	{
 		new_server = atoi(next_arg(line, &line));
 		set_server_password(new_server, line);
-		connect_to_new_server(new_server, new_server, 1);
+		reconnect(new_server);
 	}
 }
 
@@ -1612,6 +1667,7 @@ void 	server_is_connected (int sic_index, int value)
 	server_list[sic_index].rejoined_channels = 0;
 	if (value)
 	{
+		server_list[sic_index].reconnect_to = sic_index;
 		server_list[sic_index].eof = 0;
 		clear_reconnect_counts();
 	}
@@ -1658,8 +1714,8 @@ BUILT_IN_COMMAND(disconnectcmd)
 			message = args;
 
 		say("Disconnecting from server %s", get_server_itsname(i));
-		close_server(i, message);
-		window_check_servers();
+		send_to_server("QUIT :%s", message);
+		server_reconnects_to(i, -1);
 	}
 
 	if (!connected_to_server)
@@ -1675,11 +1731,44 @@ int 	auto_reconnect_callback (void *d)
 	servref = my_atol(stuff);
 	new_free((char **)&d);
 
-	get_connected(servref, servref);
+	server_reconnects_to(servref, servref);
+	reconnect(servref);
 	return 0;
 }
 
+int	server_reconnects_to (int oldref, int newref)
+{
+	if (oldref == -1)
+	{
+		reconnects_to_hint = newref;
+		return 1;
+	}
+	if (oldref < 0 || oldref >= number_of_servers)
+		return 0;
+	if (newref >= number_of_servers)
+		newref = 0;
+	if (newref < -1)
+		return 0;
+	server_list[oldref].reconnect_to = newref;
+	return 1;
+}
 
+int	set_server_quit_message (int servref, const char *message)
+{
+	if (servref < 0 || servref >= number_of_servers)
+		return -1;
+	malloc_strcpy(&server_list[servref].quit_message, message);
+	return 0;
+}
+
+const char *	get_server_quit_message (int servref)
+{
+	if (servref < 0 || servref >= number_of_servers ||
+			server_list[servref].quit_message == NULL)
+		return "get_server_quit_message";
+
+	return server_list[servref].quit_message;
+}
 
 /* PORTS */
 /* get_server_port: Returns the connection port for the given server index */
