@@ -639,6 +639,9 @@ int	fake_who_end (char *from, char *who_target)
 
 	if (who_target != NULL)
 	{
+		while (last_char(who_target) == ' ')
+			chop(who_target, 1);
+
 		/*
 		 * So 'who_target' isn't NULL here.  Make sure it's a 
 		 * legitimate match to our current top of who request.
